@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2020 Firejail Authors
+ * Copyright (C) 2014-2021 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -23,30 +23,6 @@
 static FileDB *db_skip = NULL;
 static FileDB *db_out = NULL;
 
-static void load_whitelist_common(void) {
-	FILE *fp = fopen(SYSCONFDIR "/whitelist-common.inc", "r");
-	if (!fp) {
-		fprintf(stderr, "Error: cannot open whitelist-common.inc\n");
-		exit(1);
-	}
-
-	char buf[MAX_BUF];
-	while (fgets(buf, MAX_BUF, fp)) {
-		if (strncmp(buf, "whitelist ${HOME}/", 18) != 0)
-			continue;
-		char *fn = buf + 18;
-		char *ptr = strchr(buf, '\n');
-		if (!ptr)
-			continue;
-		*ptr = '\0';
-
-		// add the file to skip list
-		db_skip = filedb_add(db_skip, fn);
-	}
-
-	fclose(fp);
-}
-
 void process_home(const char *fname, char *home, int home_len) {
 	assert(fname);
 	assert(home);
@@ -141,7 +117,7 @@ void process_home(const char *fname, char *home, int home_len) {
 		}
 
 		// skip files and directories in whitelist-common.inc
-		if (filedb_find(db_skip, toadd)) {
+		if (strlen(toadd) == 0 || filedb_find(db_skip, toadd)) {
 			if (dir)
 				free(dir);
 			continue;
@@ -162,7 +138,7 @@ void build_home(const char *fname, FILE *fp) {
 	assert(fname);
 
 	// load whitelist common
-	load_whitelist_common();
+	db_skip = filedb_load_whitelist(db_skip, "whitelist-common.inc", "whitelist ${HOME}/");
 
 	// find user home directory
 	struct passwd *pw = getpwuid(getuid());
