@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2020 Firejail Authors
+ * Copyright (C) 2014-2021 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -20,6 +20,7 @@
 
 #ifdef HAVE_CHROOT
 #include "firejail.h"
+#include "../include/gcov_wrapper.h"
 #include <sys/mount.h>
 #include <sys/sendfile.h>
 #include <errno.h>
@@ -29,7 +30,6 @@
 #define O_PATH 010000000
 #endif
 
-
 // exit if error
 void fs_check_chroot_dir(void) {
 	EUID_ASSERT();
@@ -131,9 +131,9 @@ void fs_chroot(const char *rootdir) {
 	assert(rootdir);
 
 	// fails if there is any symlink or if rootdir is not a directory
-	int parentfd = safe_fd(rootdir, O_PATH|O_DIRECTORY|O_NOFOLLOW|O_CLOEXEC);
+	int parentfd = safer_openat(-1, rootdir, O_PATH|O_DIRECTORY|O_NOFOLLOW|O_CLOEXEC);
 	if (parentfd == -1)
-		errExit("safe_fd");
+		errExit("safer_openat");
 	// rootdir has to be owned by root and is not allowed to be generally writable,
 	// this also excludes /tmp and friends
 	struct stat s;
@@ -163,30 +163,36 @@ void fs_chroot(const char *rootdir) {
 	int fd = openat(parentfd, "dev", O_PATH|O_DIRECTORY|O_NOFOLLOW|O_CLOEXEC);
 	if (fd == -1)
 		errExit("open");
-	char *proc;
-	if (asprintf(&proc, "/proc/self/fd/%d", fd) == -1)
-		errExit("asprintf");
-	if (mount("/dev", proc, NULL, MS_BIND|MS_REC, NULL) < 0)
+	if (bind_mount_path_to_fd("/dev", fd))
 		errExit("mounting /dev");
-	free(proc);
 	close(fd);
 
-	// x11
+#ifdef HAVE_X11
 	// if users want this mount, they should set FIREJAIL_CHROOT_X11
-	if (getenv("FIREJAIL_X11") || getenv("FIREJAIL_CHROOT_X11")) {
+	if (env_get("FIREJAIL_X11") || env_get("FIREJAIL_CHROOT_X11")) {
 		if (arg_debug)
 			printf("Mounting /tmp/.X11-unix on chroot /tmp/.X11-unix\n");
+		struct stat s1, s2;
+		if (stat("/tmp", &s1) || lstat("/tmp/.X11-unix", &s2))
+			errExit("mounting /tmp/.X11-unix");
+		if ((s1.st_mode & S_ISVTX) != S_ISVTX) {
+			fprintf(stderr, "Error: sticky bit not set on /tmp directory\n");
+			exit(1);
+		}
+		if (s2.st_uid != 0) {
+			fprintf(stderr, "Error: /tmp/.X11-unix not owned by root user\n");
+			exit(1);
+		}
+
 		check_subdir(parentfd, "tmp/.X11-unix", 0);
 		fd = openat(parentfd, "tmp/.X11-unix", O_PATH|O_DIRECTORY|O_NOFOLLOW|O_CLOEXEC);
 		if (fd == -1)
 			errExit("open");
-		if (asprintf(&proc, "/proc/self/fd/%d", fd) == -1)
-			errExit("asprintf");
-		if (mount("/tmp/.X11-unix", proc, NULL, MS_BIND|MS_REC, NULL) < 0)
+		if (bind_mount_path_to_fd("/tmp/.X11-unix", fd))
 			errExit("mounting /tmp/.X11-unix");
-		free(proc);
 		close(fd);
 	}
+#endif // HAVE_X11
 
 	// some older distros don't have a /run directory, create one by default
 	if (mkdirat(parentfd, "run", 0755) == -1 && errno != EEXIST)
@@ -194,7 +200,7 @@ void fs_chroot(const char *rootdir) {
 	check_subdir(parentfd, "run", 1);
 
 	// pulseaudio; only support for default directory /run/user/$UID/pulse
-	if (getenv("FIREJAIL_CHROOT_PULSE")) {
+	if (env_get("FIREJAIL_CHROOT_PULSE")) {
 		char *pulse;
 		if (asprintf(&pulse, "%s/run/user/%d/pulse", cfg.chrootdir, getuid()) == -1)
 			errExit("asprintf");
@@ -202,29 +208,21 @@ void fs_chroot(const char *rootdir) {
 
 		if (arg_debug)
 			printf("Mounting %s on chroot %s\n", orig_pulse, orig_pulse);
-		int src = safe_fd(orig_pulse, O_PATH|O_DIRECTORY|O_NOFOLLOW|O_CLOEXEC);
+		int src = safer_openat(-1, orig_pulse, O_PATH|O_DIRECTORY|O_NOFOLLOW|O_CLOEXEC);
 		if (src == -1) {
 			fprintf(stderr, "Error: cannot open %s\n", orig_pulse);
 			exit(1);
 		}
-		int dst = safe_fd(pulse, O_PATH|O_DIRECTORY|O_NOFOLLOW|O_CLOEXEC);
+		int dst = safer_openat(-1, pulse, O_PATH|O_DIRECTORY|O_NOFOLLOW|O_CLOEXEC);
 		if (dst == -1) {
 			fprintf(stderr, "Error: cannot open %s\n", pulse);
 			exit(1);
 		}
-		free(pulse);
-
-		char *proc_src, *proc_dst;
-		if (asprintf(&proc_src, "/proc/self/fd/%d", src) == -1)
-			errExit("asprintf");
-		if (asprintf(&proc_dst, "/proc/self/fd/%d", dst) == -1)
-			errExit("asprintf");
-		if (mount(proc_src, proc_dst, NULL, MS_BIND|MS_REC, NULL) < 0)
-			errExit("mount bind");
-		free(proc_src);
-		free(proc_dst);
+		if (bind_mount_by_fd(src, dst))
+			errExit("mounting pulseaudio");
 		close(src);
 		close(dst);
+		free(pulse);
 
 		// update /etc/machine-id in chroot
 		update_file(parentfd, "etc/machine-id");
@@ -243,11 +241,8 @@ void fs_chroot(const char *rootdir) {
 	fd = openat(parentfd, &RUN_FIREJAIL_LIB_DIR[1], O_PATH|O_DIRECTORY|O_NOFOLLOW|O_CLOEXEC);
 	if (fd == -1)
 		errExit("open");
-	if (asprintf(&proc, "/proc/self/fd/%d", fd) == -1)
-		errExit("asprintf");
-	if (mount(RUN_FIREJAIL_LIB_DIR, proc, NULL, MS_BIND|MS_REC, NULL) < 0)
+	if (bind_mount_path_to_fd(RUN_FIREJAIL_LIB_DIR, fd))
 		errExit("mount bind");
-	free(proc);
 	close(fd);
 
 	// create /run/firejail/mnt directory in chroot
@@ -258,29 +253,22 @@ void fs_chroot(const char *rootdir) {
 	fd = openat(parentfd, &RUN_MNT_DIR[1], O_PATH|O_DIRECTORY|O_NOFOLLOW|O_CLOEXEC);
 	if (fd == -1)
 		errExit("open");
-	if (asprintf(&proc, "/proc/self/fd/%d", fd) == -1)
-		errExit("asprintf");
-	if (mount(RUN_MNT_DIR, proc, NULL, MS_BIND|MS_REC, NULL) < 0)
+	if (bind_mount_path_to_fd(RUN_MNT_DIR, fd))
 		errExit("mount bind");
-	free(proc);
 	close(fd);
 
 	// update chroot resolv.conf
 	update_file(parentfd, "etc/resolv.conf");
 
-#ifdef HAVE_GCOV
 	__gcov_flush();
-#endif
+
 	// create /run/firejail/mnt/oroot
 	char *oroot = RUN_OVERLAY_ROOT;
 	if (mkdir(oroot, 0755) == -1)
 		errExit("mkdir");
 	// mount the chroot dir on top of /run/firejail/mnt/oroot in order to reuse the apparmor rules for overlay
-	if (asprintf(&proc, "/proc/self/fd/%d", parentfd) == -1)
-		errExit("asprintf");
-	if (mount(proc, oroot, NULL, MS_BIND|MS_REC, NULL) < 0)
+	if (bind_mount_fd_to_path(parentfd, oroot))
 		errExit("mounting rootdir oroot");
-	free(proc);
 	close(parentfd);
 	// chroot into the new directory
 	if (arg_debug)
