@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2020 Firejail Authors
+ * Copyright (C) 2014-2021 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -111,7 +111,7 @@ static int check_object_path(const char *path) {
 		}
 		++p;
 	}
-	return in_segment && segments >= 2;
+	return in_segment && segments >= 1;
 }
 
 int dbus_check_name(const char *name) {
@@ -258,12 +258,8 @@ static char *find_user_socket_by_format(char *format) {
 	if (asprintf(&dbus_user_socket, format, (int) getuid()) == -1)
 		errExit("asprintf");
 	struct stat s;
-	if (stat(dbus_user_socket, &s) == -1) {
-		if (errno == ENOENT)
-			goto fail;
-		return NULL;
-		errExit("stat");
-	}
+	if (lstat(dbus_user_socket, &s) == -1)
+		goto fail;
 	if (!S_ISSOCK(s.st_mode))
 		goto fail;
 	return dbus_user_socket;
@@ -329,7 +325,7 @@ void dbus_proxy_start(void) {
 			errExit("close");
 
 		if (arg_dbus_user == DBUS_POLICY_FILTER) {
-			char *user_env = getenv(DBUS_SESSION_BUS_ADDRESS_ENV);
+			const char *user_env = env_get(DBUS_SESSION_BUS_ADDRESS_ENV);
 			if (user_env == NULL) {
 				char *dbus_user_socket = find_user_socket();
 				write_arg(args_pipe[1], DBUS_SOCKET_PATH_PREFIX "%s",
@@ -350,7 +346,7 @@ void dbus_proxy_start(void) {
 		}
 
 		if (arg_dbus_system == DBUS_POLICY_FILTER) {
-			char *system_env = getenv(DBUS_SYSTEM_BUS_ADDRESS_ENV);
+			const char *system_env = env_get(DBUS_SYSTEM_BUS_ADDRESS_ENV);
 			if (system_env == NULL) {
 				write_arg(args_pipe[1],
 						  DBUS_SOCKET_PATH_PREFIX DBUS_SYSTEM_SOCKET);
@@ -416,7 +412,7 @@ void dbus_proxy_stop(void) {
 }
 
 static void socket_overlay(char *socket_path, char *proxy_path) {
-	int fd = safe_fd(proxy_path, O_PATH | O_NOFOLLOW | O_CLOEXEC);
+	int fd = safer_openat(-1, proxy_path, O_PATH | O_NOFOLLOW | O_CLOEXEC);
 	if (fd == -1)
 		errExit("opening DBus proxy socket");
 	struct stat s;
@@ -426,17 +422,13 @@ static void socket_overlay(char *socket_path, char *proxy_path) {
 		errno = ENOTSOCK;
 		errExit("mounting DBus proxy socket");
 	}
-	char *proxy_fd_path;
-	if (asprintf(&proxy_fd_path, "/proc/self/fd/%d", fd) == -1)
-		errExit("asprintf");
-	if (mount(proxy_path, socket_path, NULL, MS_BIND | MS_REC, NULL) == -1)
+	if (bind_mount_fd_to_path(fd, socket_path))
 		errExit("mount bind");
-	free(proxy_fd_path);
 	close(fd);
 }
 
-static char *get_socket_env(const char *name) {
-	char *value = getenv(name);
+static const char *get_socket_env(const char *name) {
+	const char *value = env_get(name);
 	if (value == NULL)
 		return NULL;
 	if (strncmp(value, DBUS_SOCKET_PATH_PREFIX,
@@ -446,21 +438,13 @@ static char *get_socket_env(const char *name) {
 }
 
 void dbus_set_session_bus_env(void) {
-	if (setenv(DBUS_SESSION_BUS_ADDRESS_ENV,
-			DBUS_SOCKET_PATH_PREFIX RUN_DBUS_USER_SOCKET, 1) == -1) {
-		fprintf(stderr, "Error: cannot modify " DBUS_SESSION_BUS_ADDRESS_ENV
-						" required by --dbus-user\n");
-		exit(1);
-	}
+	env_store_name_val(DBUS_SESSION_BUS_ADDRESS_ENV,
+			   DBUS_SOCKET_PATH_PREFIX RUN_DBUS_USER_SOCKET, SETENV);
 }
 
 void dbus_set_system_bus_env(void) {
-	if (setenv(DBUS_SYSTEM_BUS_ADDRESS_ENV,
-			DBUS_SOCKET_PATH_PREFIX RUN_DBUS_SYSTEM_SOCKET, 1) == -1) {
-		fprintf(stderr, "Error: cannot modify " DBUS_SYSTEM_BUS_ADDRESS_ENV
-						" required by --dbus-system\n");
-		exit(1);
-	}
+	env_store_name_val(DBUS_SYSTEM_BUS_ADDRESS_ENV,
+			   DBUS_SOCKET_PATH_PREFIX RUN_DBUS_SYSTEM_SOCKET, SETENV);
 }
 
 static void disable_socket_dir(void) {
@@ -486,7 +470,7 @@ void dbus_apply_policy(void) {
 	create_empty_dir_as_root(RUN_DBUS_DIR, 0755);
 
 	if (arg_dbus_user != DBUS_POLICY_ALLOW) {
-		create_empty_file_as_root(RUN_DBUS_USER_SOCKET, 0700);
+		create_empty_file_as_root(RUN_DBUS_USER_SOCKET, 0600);
 
 		if (arg_dbus_user == DBUS_POLICY_FILTER) {
 			assert(dbus_user_proxy_socket != NULL);
@@ -506,7 +490,7 @@ void dbus_apply_policy(void) {
 			errExit("asprintf");
 		disable_file_or_dir(dbus_user_socket2);
 
-		char *user_env = get_socket_env(DBUS_SESSION_BUS_ADDRESS_ENV);
+		const char *user_env = get_socket_env(DBUS_SESSION_BUS_ADDRESS_ENV);
 		if (user_env != NULL && strcmp(user_env, dbus_user_socket) != 0 &&
 			strcmp(user_env, dbus_user_socket2) != 0)
 			disable_file_or_dir(user_env);
@@ -525,7 +509,7 @@ void dbus_apply_policy(void) {
 	}
 
 	if (arg_dbus_system != DBUS_POLICY_ALLOW) {
-		create_empty_file_as_root(RUN_DBUS_SYSTEM_SOCKET, 0700);
+		create_empty_file_as_root(RUN_DBUS_SYSTEM_SOCKET, 0600);
 
 		if (arg_dbus_system == DBUS_POLICY_FILTER) {
 			assert(dbus_system_proxy_socket != NULL);
@@ -535,7 +519,7 @@ void dbus_apply_policy(void) {
 
 		disable_file_or_dir(DBUS_SYSTEM_SOCKET);
 
-		char *system_env = get_socket_env(DBUS_SYSTEM_BUS_ADDRESS_ENV);
+		const char *system_env = get_socket_env(DBUS_SYSTEM_BUS_ADDRESS_ENV);
 		if (system_env != NULL && strcmp(system_env, DBUS_SYSTEM_SOCKET) != 0)
 			disable_file_or_dir(system_env);
 
@@ -561,4 +545,4 @@ void dbus_apply_policy(void) {
 
 	fwarning("An abstract unix socket for session D-BUS might still be available. Use --net or remove unix from --protocol set.\n");
 }
-#endif // HAVE_DBUSPROXY
\ No newline at end of file
+#endif // HAVE_DBUSPROXY
