@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2020 Firejail Authors
+ * Copyright (C) 2014-2021 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -18,6 +18,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 #include "firejail.h"
+#include <errno.h>
 #include <sys/mount.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -51,7 +52,7 @@ void fs_machineid(void) {
 	mid.u8[8] = (mid.u8[8] & 0x3F) | 0x80;
 
 	// write it in a file
-	FILE *fp = fopen(RUN_MACHINEID, "w");
+	FILE *fp = fopen(RUN_MACHINEID, "we");
 	if (!fp)
 		errExit("fopen");
 	fprintf(fp, "%08x%08x%08x%08x\n", mid.u32[0], mid.u32[1], mid.u32[2], mid.u32[3]);
@@ -75,6 +76,44 @@ void fs_machineid(void) {
 	}
 }
 
+// Duplicate directory structure from src to dst by creating empty directories.
+// The paths _must_ be identical after their respective prefixes.
+// When finished, dst will point to the target directory. That is, if
+// it starts out pointing to a file, it will instead be truncated so
+// that it contains the parent directory instead.
+static void build_dirs(char *src, char *dst, size_t src_prefix_len, size_t dst_prefix_len) {
+	char *p = src + src_prefix_len + 1;
+	char *q = dst + dst_prefix_len + 1;
+	char *r = dst + dst_prefix_len;
+	struct stat s;
+	bool last = false;
+	*r = '\0';
+	for (; !last; p++, q++) {
+		if (*p == '\0') {
+			last = true;
+		}
+		if (*p == '\0' || (*p == '/' && *(p - 1) != '/')) {
+			// We found a new component of our src path.
+			// Null-terminate it temporarily here so that we can work
+			// with it.
+			*p = '\0';
+			if (stat(src, &s) == 0 && S_ISDIR(s.st_mode)) {
+				// Null-terminate the dst path and undo its previous
+				// termination.
+				*q = '\0';
+				*r = '/';
+				r = q;
+				create_empty_dir_as_root(dst, s.st_mode);
+			}
+			if (!last) {
+				// If we're not at the final terminating null, restore
+				// the slash so that we can continue our traversal.
+				*p = '/';
+			}
+		}
+	}
+}
+
 // return 0 if file not found, 1 if found
 static int check_dir_or_file(const char *fname) {
 	assert(fname);
@@ -102,7 +141,7 @@ errexit:
 static void duplicate(const char *fname, const char *private_dir, const char *private_run_dir) {
 	assert(fname);
 
-	if (*fname == '~' || strchr(fname, '/') || strcmp(fname, "..") == 0) {
+	if (*fname == '~' || *fname == '/' || strncmp(fname, "..", 2) == 0) {
 		fprintf(stderr, "Error: \"%s\" is an invalid filename\n", fname);
 		exit(1);
 	}
@@ -118,27 +157,22 @@ static void duplicate(const char *fname, const char *private_dir, const char *pr
 	}
 
 	if (arg_debug)
-		printf("copying %s to private %s\n", src, private_dir);
+		printf("Copying %s to private %s\n", src, private_dir);
 
-	struct stat s;
-	if (stat(src, &s) == 0 && S_ISDIR(s.st_mode)) {
-		// create the directory in RUN_ETC_DIR
-		char *dirname;
-		if (asprintf(&dirname, "%s/%s", private_run_dir, fname) == -1)
-			errExit("asprintf");
-		create_empty_dir_as_root(dirname, s.st_mode);
-		sbox_run(SBOX_ROOT| SBOX_SECCOMP, 3, PATH_FCOPY, src, dirname);
-		free(dirname);
-	}
-	else
-		sbox_run(SBOX_ROOT| SBOX_SECCOMP, 3, PATH_FCOPY, src, private_run_dir);
+	char *dst;
+	if (asprintf(&dst, "%s/%s", private_run_dir, fname) == -1)
+		errExit("asprintf");
 
+	build_dirs(src, dst, strlen(private_dir), strlen(private_run_dir));
+	sbox_run(SBOX_ROOT | SBOX_SECCOMP, 3, PATH_FCOPY, src, dst);
+
+	free(dst);
 	fs_logger2("clone", src);
 	free(src);
 }
 
 
-void fs_private_dir_list(const char *private_dir, const char *private_run_dir, const char *private_list) {
+void fs_private_dir_copy(const char *private_dir, const char *private_run_dir, const char *private_list) {
 	assert(private_dir);
 	assert(private_run_dir);
 	assert(private_list);
@@ -147,12 +181,10 @@ void fs_private_dir_list(const char *private_dir, const char *private_run_dir, c
 	struct stat s;
 	if (stat(private_dir, &s) == -1) {
 		if (arg_debug)
-			printf("Cannot find %s\n", private_dir);
+			printf("Cannot find %s: %s\n", private_dir, strerror(errno));
 		return;
 	}
 
-	timetrace_start();
-
 	// create /run/firejail/mnt/etc directory
 	mkdir_attr(private_run_dir, 0755, 0, 0);
 	selinux_relabel_path(private_run_dir, private_dir);
@@ -185,9 +217,23 @@ void fs_private_dir_list(const char *private_dir, const char *private_run_dir, c
 		free(dlist);
 		fs_logger_print();
 	}
+}
+
+void fs_private_dir_mount(const char *private_dir, const char *private_run_dir) {
+	assert(private_dir);
+	assert(private_run_dir);
 
 	if (arg_debug)
 		printf("Mount-bind %s on top of %s\n", private_run_dir, private_dir);
+
+	// nothing to do if directory does not exist
+	struct stat s;
+	if (stat(private_dir, &s) == -1) {
+		if (arg_debug)
+			printf("Cannot find %s: %s\n", private_dir, strerror(errno));
+		return;
+	}
+
 	if (mount(private_run_dir, private_dir, NULL, MS_BIND|MS_REC, NULL) < 0)
 		errExit("mount bind");
 	fs_logger2("mount", private_dir);
@@ -196,6 +242,11 @@ void fs_private_dir_list(const char *private_dir, const char *private_run_dir, c
 	if (mount("tmpfs", private_run_dir, "tmpfs", MS_NOSUID | MS_NODEV | MS_STRICTATIME,  "mode=755,gid=0") < 0)
 		errExit("mounting tmpfs");
 	fs_logger2("tmpfs", private_run_dir);
+}
 
+void fs_private_dir_list(const char *private_dir, const char *private_run_dir, const char *private_list) {
+	timetrace_start();
+	fs_private_dir_copy(private_dir, private_run_dir, private_list);
+	fs_private_dir_mount(private_dir, private_run_dir);
 	fmessage("Private %s installed in %0.2f ms\n", private_dir, timetrace_end());
 }
