@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2020 Firejail Authors
+ * Copyright (C) 2014-2021 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -23,16 +23,44 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
-#include <dirent.h>
+#include <fcntl.h>
+#include <errno.h>
 #include <glob.h>
 #define MAXBUF 4096
 
 extern void fslib_install_stdc(void);
+extern void fslib_install_firejail(void);
 extern void fslib_install_system(void);
 
 static int lib_cnt = 0;
 static int dir_cnt = 0;
 
+static const char *masked_lib_dirs[] = {
+	"/usr/lib64",
+	"/lib64",
+	"/usr/lib",
+	"/lib",
+	"/usr/local/lib64",
+	"/usr/local/lib",
+	NULL,
+};
+
+// return 1 if the file is in masked_lib_dirs[]
+static int valid_full_path(const char *full_path) {
+	if (strstr(full_path, ".."))
+		return 0;
+
+	int i = 0;
+	while (masked_lib_dirs[i]) {
+		size_t len = strlen(masked_lib_dirs[i]);
+		if (strncmp(full_path, masked_lib_dirs[i], len) == 0 &&
+		    full_path[len] == '/')
+			return 1;
+		i++;
+	}
+	return 0;
+}
+
 char *find_in_path(const char *program) {
 	EUID_ASSERT();
 	if (arg_debug)
@@ -44,9 +72,10 @@ char *find_in_path(const char *program) {
 		errExit("readlink");
 	self[len] = '\0';
 
-	char *path = getenv("PATH");
+	const char *path = env_get("PATH");
 	if (!path)
 		return NULL;
+
 	char *dup = strdup(path);
 	if (!dup)
 		errExit("strdup");
@@ -79,22 +108,6 @@ char *find_in_path(const char *program) {
 	return NULL;
 }
 
-static void report_duplication(const char *full_path) {
-	char *fname = strrchr(full_path, '/');
-	if (fname && *(++fname) != '\0') {
-		// report the file on all bin paths
-		int i = 0;
-		while (default_lib_paths[i]) {
-			char *p;
-			if (asprintf(&p, "%s/%s", default_lib_paths[i], fname) == -1)
-				errExit("asprintf");
-			fs_logger2("clone", p);
-			free(p);
-			i++;
-		}
-	}
-}
-
 static char *build_dest_dir(const char *full_path) {
 	assert(full_path);
 	if (strstr(full_path, "/x86_64-linux-gnu/"))
@@ -102,71 +115,112 @@ static char *build_dest_dir(const char *full_path) {
 	return RUN_LIB_DIR;
 }
 
-// copy fname in private_run_dir
-void fslib_duplicate(const char *full_path) {
+// return name of mount target in allocated memory
+static char *build_dest_name(const char *full_path) {
 	assert(full_path);
+	char *fname = strrchr(full_path, '/');
+	assert(fname);
+	fname++;
+	// no trailing slash or dot
+	assert(fname[0] != '\0' && (fname[0] != '.' || fname[1] != '\0'));
 
-	struct stat s;
-	if (stat(full_path, &s) != 0 || s.st_uid != 0 || access(full_path, R_OK))
-		return;
-
-	char *dest_dir = build_dest_dir(full_path);
-
-	// don't copy it if the file is already there
-	char *ptr = strrchr(full_path, '/');
-	if (!ptr)
-		return;
-	ptr++;
-	if (*ptr == '\0')
-		return;
-
-	char *name;
-	if (asprintf(&name, "%s/%s", dest_dir, ptr) == -1)
+	char *dest;
+	if (asprintf(&dest, "%s/%s", build_dest_dir(full_path), fname) == -1)
 		errExit("asprintf");
-	if (stat(name, &s) == 0) {
-		free(name);
-		return;
+	return dest;
+}
+
+static void fslib_mount_dir(const char *full_path) {
+	// create new directory and mount the original on top of it
+	char *dest = build_dest_name(full_path);
+	if (mkdir(dest, 0755) == -1) {
+		if (errno == EEXIST) { // directory has been mounted already, nothing to do
+			free(dest);
+			return;
+		}
+		errExit("mkdir");
 	}
-	free(name);
 
 	if (arg_debug || arg_debug_private_lib)
-		printf("    copying %s to private %s\n", full_path, dest_dir);
+		printf("    mounting %s on %s\n", full_path, dest);
+	// if full_path is a symbolic link, mount will follow it
+	if (mount(full_path, dest, NULL, MS_BIND|MS_REC, NULL) < 0)
+		errExit("mount bind");
+	free(dest);
+	dir_cnt++;
+}
 
-	sbox_run(SBOX_ROOT| SBOX_SECCOMP, 4, PATH_FCOPY, "--follow-link", full_path, dest_dir);
-	report_duplication(full_path);
+static void fslib_mount_file(const char *full_path) {
+	// create new file and mount the original on top of it
+	char *dest = build_dest_name(full_path);
+	int fd = open(dest, O_RDONLY|O_CREAT|O_EXCL|O_CLOEXEC, S_IRUSR | S_IWUSR);
+	if (fd == -1) {
+		if (errno == EEXIST) { // file has been mounted already, nothing to do
+			free(dest);
+			return;
+		}
+		errExit("open");
+	}
+	close(fd);
+
+	if (arg_debug || arg_debug_private_lib)
+		printf("    mounting %s on %s\n", full_path, dest);
+	// if full_path is a symbolic link, mount will follow it
+	if (mount(full_path, dest, NULL, MS_BIND, NULL) < 0)
+		errExit("mount bind");
+	free(dest);
 	lib_cnt++;
 }
 
+void fslib_mount(const char *full_path) {
+	assert(full_path);
+	struct stat s;
+
+	if (*full_path == '\0' ||
+	    !valid_full_path(full_path) ||
+	    stat_as_user(full_path, &s) != 0 ||
+	    s.st_uid != 0)
+		return;
+
+	if (S_ISDIR(s.st_mode))
+		fslib_mount_dir(full_path);
+	else if (S_ISREG(s.st_mode) && is_lib_64(full_path))
+		fslib_mount_file(full_path);
+}
 
 // requires full path for lib
 // it could be a library or an executable
 // lib is not copied, only libraries used by it
-void fslib_copy_libs(const char *full_path) {
+void fslib_mount_libs(const char *full_path, unsigned user) {
 	assert(full_path);
-	if (arg_debug || arg_debug_private_lib)
-		printf("    fslib_copy_libs %s\n", full_path);
-
 	// if library/executable does not exist or the user does not have read access to it
 	// print a warning and exit the function.
-	if (access(full_path, R_OK)) {
+	if (user && access(full_path, R_OK)) {
 		if (arg_debug || arg_debug_private_lib)
-			printf("cannot find %s for private-lib, skipping...\n", full_path);
+			printf("Cannot read %s, skipping...\n", full_path);
 		return;
 	}
 
+	if (arg_debug || arg_debug_private_lib)
+		printf("    fslib_mount_libs %s\n", full_path);
 	// create an empty RUN_LIB_FILE and allow the user to write to it
 	unlink(RUN_LIB_FILE);			  // in case is there
 	create_empty_file_as_root(RUN_LIB_FILE, 0644);
-	if (chown(RUN_LIB_FILE, getuid(), getgid()))
+	if (user && chown(RUN_LIB_FILE, getuid(), getgid()))
 		errExit("chown");
 
 	// run fldd to extract the list of files
 	if (arg_debug || arg_debug_private_lib)
-		printf("    running fldd %s\n", full_path);
-	sbox_run(SBOX_USER | SBOX_SECCOMP | SBOX_CAPS_NONE, 3, PATH_FLDD, full_path, RUN_LIB_FILE);
+		printf("    running fldd %s as %s\n", full_path, user ? "user" : "root");
+	unsigned mask;
+	if (user)
+		mask = SBOX_USER;
+	else
+		mask = SBOX_ROOT;
+	sbox_run(mask | SBOX_SECCOMP | SBOX_CAPS_NONE, 3, PATH_FLDD, full_path, RUN_LIB_FILE);
 
 	// open the list of libraries and install them on by one
-	FILE *fp = fopen(RUN_LIB_FILE, "r");
+	FILE *fp = fopen(RUN_LIB_FILE, "re");
 	if (!fp)
 		errExit("fopen");
 
@@ -176,68 +230,31 @@ void fslib_copy_libs(const char *full_path) {
 		char *ptr = strchr(buf, '\n');
 		if (ptr)
 			*ptr = '\0';
-		fslib_duplicate(buf);
+
+		trim_trailing_slash_or_dot(buf);
+		fslib_mount(buf);
 	}
 	fclose(fp);
 	unlink(RUN_LIB_FILE);
 }
 
-
-void fslib_copy_dir(const char *full_path) {
-	assert(full_path);
-	if (arg_debug || arg_debug_private_lib)
-		printf("    fslib_copy_dir %s\n", full_path);
-
-	// do nothing if the directory does not exist or is not owned by root
-	struct stat s;
-	if (stat(full_path, &s) != 0 || s.st_uid != 0 || !S_ISDIR(s.st_mode) || access(full_path, R_OK))
-		return;
-
-	char *dir_name = strrchr(full_path, '/');
-	assert(dir_name);
-	dir_name++;
-	assert(*dir_name != '\0');
-
-	// do nothing if the directory is already there
-	char *dest;
-	if (asprintf(&dest, "%s/%s", build_dest_dir(full_path), dir_name) == -1)
-		errExit("asprintf");
-	if (stat(dest, &s) == 0) {
-		free(dest);
-		return;
-	}
-
-	// create new directory and mount the original on top of it
-	mkdir_attr(dest, 0755, 0, 0);
-
-	if (mount(full_path, dest, NULL, MS_BIND|MS_REC, NULL) < 0 ||
-		mount(NULL, dest, NULL, MS_BIND|MS_REMOUNT|MS_RDONLY|MS_NOSUID|MS_NODEV|MS_REC, NULL) < 0)
-		errExit("mount bind");
-	fs_logger2("clone", full_path);
-	fs_logger2("mount", full_path);
-	dir_cnt++;
-	free(dest);
-}
-
-// fname should be a vallid full path at this point
+// fname should be a full path at this point
 static void load_library(const char *fname) {
 	assert(fname);
 	assert(*fname == '/');
 
-	// existing file owned by root, read access
+	// existing file owned by root
 	struct stat s;
-	if (stat(fname, &s) == 0 && s.st_uid == 0 && !access(fname, R_OK)) {
+	if (stat_as_user(fname, &s) == 0 && s.st_uid == 0) {
 		// load directories, regular 64 bit libraries, and 64 bit executables
-		if (is_dir(fname) || is_lib_64(fname)) {
-			if (is_dir(fname))
-				fslib_copy_dir(fname);
-			else {
-				if (strstr(fname, ".so") ||
-				    access(fname, X_OK) != 0) // don't duplicate executables, just install the libraries
-					fslib_duplicate(fname);
+		if (S_ISDIR(s.st_mode))
+			fslib_mount(fname);
+		else if (S_ISREG(s.st_mode) && is_lib_64(fname)) {
+			if (strstr(fname, ".so") ||
+			    access(fname, X_OK) != 0) // don't duplicate executables, just install the libraries
+				fslib_mount(fname);
 
-				fslib_copy_libs(fname);
-			}
+			fslib_mount_libs(fname, 1); // parse as user
 		}
 	}
 }
@@ -268,17 +285,24 @@ static void install_list_entry(const char *lib) {
 #define DO_GLOBBING
 #ifdef DO_GLOBBING
 		// globbing
+		EUID_USER();
 		glob_t globbuf;
 		int globerr = glob(fname, GLOB_NOCHECK | GLOB_NOSORT | GLOB_PERIOD, NULL, &globbuf);
 		if (globerr) {
 			fprintf(stderr, "Error: failed to glob private-lib pattern %s\n", fname);
 			exit(1);
 		}
+		EUID_ROOT();
 		size_t j;
 		for (j = 0; j < globbuf.gl_pathc; j++) {
 			assert(globbuf.gl_pathv[j]);
 //printf("glob %s\n", globbuf.gl_pathv[j]);
 			// GLOB_NOCHECK - no pattern matched returns the original pattern; try to load it anyway
+
+			// foobar/* expands to foobar/. and foobar/..
+			const char *base = gnu_basename(globbuf.gl_pathv[j]);
+			if (strcmp(base, ".") == 0 || strcmp(base, "..") == 0)
+				continue;
 			load_library(globbuf.gl_pathv[j]);
 		}
 
@@ -293,7 +317,6 @@ static void install_list_entry(const char *lib) {
 	return;
 }
 
-
 void fslib_install_list(const char *lib_list) {
 	assert(lib_list);
 	if (arg_debug || arg_debug_private_lib)
@@ -308,42 +331,31 @@ void fslib_install_list(const char *lib_list) {
 		fprintf(stderr, "Error: invalid private-lib argument\n");
 		exit(1);
 	}
+	trim_trailing_slash_or_dot(ptr);
 	install_list_entry(ptr);
 
-	while ((ptr = strtok(NULL, ",")) != NULL)
+	while ((ptr = strtok(NULL, ",")) != NULL) {
+		trim_trailing_slash_or_dot(ptr);
 		install_list_entry(ptr);
+	}
 	free(dlist);
 	fs_logger_print();
 }
 
-
-
 static void mount_directories(void) {
-	if (arg_debug || arg_debug_private_lib)
-		printf("Mount-bind %s on top of /lib /lib64 /usr/lib\n", RUN_LIB_DIR);
+	fs_remount(RUN_LIB_DIR, MOUNT_READONLY, 1); // should be redundant except for RUN_LIB_DIR itself
 
-	if (is_dir("/lib")) {
-		if (mount(RUN_LIB_DIR, "/lib", NULL, MS_BIND|MS_REC, NULL) < 0 ||
-			mount(NULL, "/lib", NULL, MS_BIND|MS_REMOUNT|MS_NOSUID|MS_NODEV|MS_REC, NULL) < 0)
-			errExit("mount bind");
-		fs_logger2("tmpfs", "/lib");
-		fs_logger("mount /lib");
-	}
-
-	if (is_dir("/lib64")) {
-		if (mount(RUN_LIB_DIR, "/lib64", NULL, MS_BIND|MS_REC, NULL) < 0 ||
-			mount(NULL, "/lib64", NULL, MS_BIND|MS_REMOUNT|MS_NOSUID|MS_NODEV|MS_REC, NULL) < 0)
-			errExit("mount bind");
-		fs_logger2("tmpfs", "/lib64");
-		fs_logger("mount /lib64");
-	}
-
-	if (is_dir("/usr/lib")) {
-		if (mount(RUN_LIB_DIR, "/usr/lib", NULL, MS_BIND|MS_REC, NULL) < 0 ||
-			mount(NULL, "/usr/lib", NULL, MS_BIND|MS_REMOUNT|MS_NOSUID|MS_NODEV|MS_REC, NULL) < 0)
-			errExit("mount bind");
-		fs_logger2("tmpfs", "/usr/lib");
-		fs_logger("mount /usr/lib");
+	int i = 0;
+	while (masked_lib_dirs[i]) {
+		if (is_dir(masked_lib_dirs[i])) {
+			if (arg_debug || arg_debug_private_lib)
+				printf("Mount-bind %s on top of %s\n", RUN_LIB_DIR, masked_lib_dirs[i]);
+			if (mount(RUN_LIB_DIR, masked_lib_dirs[i], NULL, MS_BIND|MS_REC, NULL) < 0)
+				errExit("mount bind");
+			fs_logger2("tmpfs", masked_lib_dirs[i]);
+			fs_logger2("mount", masked_lib_dirs[i]);
+		}
+		i++;
 	}
 
 	// for amd64 only - we'll deal with i386 later
@@ -379,25 +391,12 @@ void fs_private_lib(void) {
 		printf("Installing standard C library\n");
 	fslib_install_stdc();
 
-	// start timetrace
-	timetrace_start();
-
-	// bring in firejail executable libraries in case we are redirected here by a firejail symlink from /usr/local/bin/firejail
+	// install other libraries needed by firejail
 	if (arg_debug || arg_debug_private_lib)
 		printf("Installing Firejail libraries\n");
-	fslib_install_list(PATH_FIREJAIL);
-
-	// bring in firejail directory
-	fslib_install_list(LIBDIR "/firejail");
-
-	// bring in dhclient libraries
-	if (any_dhcp()) {
-		if (arg_debug || arg_debug_private_lib)
-			printf("Installing dhclient libraries\n");
-		fslib_install_list(RUN_MNT_DIR "/dhclient");
-	}
-	fmessage("Firejail libraries installed in %0.2f ms\n", timetrace_end());
+	fslib_install_firejail();
 
+	// start timetrace
 	timetrace_start();
 
 	// copy the libs in the new lib directory for the main exe
@@ -426,7 +425,6 @@ void fs_private_lib(void) {
 		fslib_install_list(cfg.shell);
 		// a shell is useless without some basic commands
 		fslib_install_list("/bin/ls,/bin/cat,/bin/mv,/bin/rm");
-
 	}
 
 	// for the listed libs and directories
