@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2020 Firejail Authors
+ * Copyright (C) 2014-2021 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -18,6 +18,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 #include "firejail.h"
+#include "../include/gcov_wrapper.h"
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -25,7 +26,6 @@
 #include <sys/wait.h>
 #include <string.h>
 
-
 static void check(const char *fname) {
 	// manufacture /run/user directory
 	char *runuser;
@@ -46,7 +46,7 @@ static void mkdir_recursive(char *path) {
 	struct stat s;
 
 	if (chdir("/")) {
-		fprintf(stderr, "Error: can't chdir to /");
+		fprintf(stderr, "Error: can't chdir to /\n");
 		return;
 	}
 
@@ -63,7 +63,7 @@ static void mkdir_recursive(char *path) {
 			return;
 		}
 		if (chdir(subdir)) {
-			fprintf(stderr, "Error: can't chdir to %s", subdir);
+			fprintf(stderr, "Error: can't chdir to %s\n", subdir);
 			return;
 		}
 
@@ -95,9 +95,9 @@ void fs_mkdir(const char *name) {
 
 		// create directory
 		mkdir_recursive(expanded);
-#ifdef HAVE_GCOV
+
 		__gcov_flush();
-#endif
+
 		_exit(0);
 	}
 	// wait for the child to finish
