@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2020 Firejail Authors
+ * Copyright (C) 2014-2021 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -217,7 +217,7 @@ int net_add_route(uint32_t ip, uint32_t mask, uint32_t gw) {
 
 #define BUFSIZE 1024
 uint32_t network_get_defaultgw(void) {
-	FILE *fp = fopen("/proc/self/net/route", "r");
+	FILE *fp = fopen("/proc/self/net/route", "re");
 	if (!fp)
 		errExit("fopen");
 
