@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2020 Firejail Authors
+ * Copyright (C) 2014-2021 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -26,13 +26,13 @@ static unsigned int longest_path_elt = 0;
 
 static char *elt = NULL; // moved from inside init_paths in order to get rid of scan-build warning
 static void init_paths(void) {
-	char *path = getenv("PATH");
+	const char *env_path = env_get("PATH");
 	char *p;
-	if (!path) {
-		path = "/usr/local/bin:/usr/bin:/bin:/usr/local/sbin:/usr/sbin:/sbin";
-		setenv("PATH", path, 1);
+	if (!env_path) {
+		env_path = "/usr/local/bin:/usr/bin:/bin:/usr/local/sbin:/usr/sbin:/sbin";
+		env_store_name_val("PATH", env_path, SETENV);
 	}
-	path = strdup(path);
+	char *path = strdup(env_path);
 	if (!path)
 		errExit("strdup");
 
@@ -136,7 +136,7 @@ int program_in_path(const char *program) {
 			// ('x' permission means something different for directories).
 			// exec follows symlinks, so use stat, not lstat.
 			struct stat st;
-			if (stat(scratch, &st)) {
+			if (stat_as_user(scratch, &st)) {
 				perror(scratch);
 				exit(1);
 			}
