@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2020 Firejail Authors
+ * Copyright (C) 2014-2021 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -18,10 +18,12 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 #include "firejail.h"
+#include "../include/gcov_wrapper.h"
 #include "../include/seccomp.h"
 #include "../include/syscall.h"
 #include <dirent.h>
 #include <sys/stat.h>
+
 extern char *xephyr_screen;
 
 #define MAX_READ 8192				  // line buffer for profile files
@@ -157,8 +159,12 @@ static int check_nosound(void) {
 	return arg_nosound != 0;
 }
 
+static int check_private(void) {
+	return arg_private;
+}
+
 static int check_x11(void) {
-	return (arg_x11_block || arg_x11_xorg || getenv("FIREJAIL_X11"));
+	return (arg_x11_block || arg_x11_xorg || env_get("FIREJAIL_X11"));
 }
 
 static int check_disable_u2f(void) {
@@ -174,6 +180,7 @@ Cond conditionals[] = {
 	{"HAS_NET", check_netoptions},
 	{"HAS_NODBUS", check_nodbus},
 	{"HAS_NOSOUND", check_nosound},
+	{"HAS_PRIVATE", check_private},
 	{"HAS_X11", check_x11},
 	{"BROWSER_DISABLE_U2F", check_disable_u2f},
 	{"BROWSER_ALLOW_DRM", check_allow_drm},
@@ -418,7 +425,7 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 		return 0;
 	}
 	else if (strcmp(ptr, "noautopulse") == 0) {
-		arg_noautopulse = 1;
+		arg_keep_config_pulse = 1;
 		return 0;
 	}
 	else if (strcmp(ptr, "notv") == 0) {
@@ -437,6 +444,10 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 		arg_no3d = 1;
 		return 0;
 	}
+	else if (strcmp(ptr, "noinput") == 0) {
+		arg_noinput = 1;
+		return 0;
+	}
 	else if (strcmp(ptr, "nodbus") == 0) {
 #ifdef HAVE_DBUSPROXY
 		arg_dbus_user = DBUS_POLICY_BLOCK;
@@ -911,15 +922,10 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 
 	if (strncmp(ptr, "protocol ", 9) == 0) {
 		if (checkcfg(CFG_SECCOMP)) {
-			if (cfg.protocol) {
-				fwarning("more than one protocol list is present, \"%s\" will be installed\n", cfg.protocol);
-				return 0;
-			}
-
-			// store list
-			cfg.protocol = strdup(ptr + 9);
-			if (!cfg.protocol)
-				errExit("strdup");
+			const char *add = ptr + 9;
+			profile_list_augment(&cfg.protocol, add);
+			if (arg_debug)
+				fprintf(stderr, "[profile] combined protocol list: \"%s\"\n", cfg.protocol);
 		}
 		else
 			warning_feature_disabled("seccomp");
@@ -931,7 +937,6 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 		return 0;
 	}
 	if (strncmp(ptr, "rmenv ", 6) == 0) {
-		unsetenv(ptr + 6); // Remove also immediately from Firejail itself
 		env_store(ptr + 6, RMENV);
 		return 0;
 	}
@@ -1140,6 +1145,12 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 		arg_machineid = 1;
 		return 0;
 	}
+
+	if (strcmp(ptr, "keep-config-pulse") == 0) {
+		arg_keep_config_pulse = 1;
+		return 0;
+	}
+
 	// writable-var
 	if (strcmp(ptr, "writable-var") == 0) {
 		arg_writable_var = 1;
@@ -1181,7 +1192,7 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 	if (strcmp(ptr, "x11 xephyr") == 0) {
 #ifdef HAVE_X11
 		if (checkcfg(CFG_X11)) {
-			char *x11env = getenv("FIREJAIL_X11");
+			const char *x11env = env_get("FIREJAIL_X11");
 			if (x11env && strcmp(x11env, "yes") == 0) {
 				return 0;
 			}
@@ -1210,7 +1221,7 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 	if (strcmp(ptr, "x11 xpra") == 0) {
 #ifdef HAVE_X11
 		if (checkcfg(CFG_X11)) {
-			char *x11env = getenv("FIREJAIL_X11");
+			const char *x11env = env_get("FIREJAIL_X11");
 			if (x11env && strcmp(x11env, "yes") == 0) {
 				return 0;
 			}
@@ -1229,7 +1240,7 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 	if (strcmp(ptr, "x11 xvfb") == 0) {
 #ifdef HAVE_X11
 		if (checkcfg(CFG_X11)) {
-			char *x11env = getenv("FIREJAIL_X11");
+			const char *x11env = env_get("FIREJAIL_X11");
 			if (x11env && strcmp(x11env, "yes") == 0) {
 				return 0;
 			}
@@ -1248,7 +1259,7 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 	if (strcmp(ptr, "x11") == 0) {
 #ifdef HAVE_X11
 		if (checkcfg(CFG_X11)) {
-			char *x11env = getenv("FIREJAIL_X11");
+			const char *x11env = env_get("FIREJAIL_X11");
 			if (x11env && strcmp(x11env, "yes") == 0) {
 				return 0;
 			}
@@ -1266,56 +1277,69 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 
 	// private /etc list of files and directories
 	if (strncmp(ptr, "private-etc ", 12) == 0) {
-		if (arg_writable_etc) {
-			fprintf(stderr, "Error: --private-etc and --writable-etc are mutually exclusive\n");
-			exit(1);
+		if (checkcfg(CFG_PRIVATE_ETC)) {
+			if (arg_writable_etc) {
+				fprintf(stderr, "Error: --private-etc and --writable-etc are mutually exclusive\n");
+				exit(1);
+			}
+			if (cfg.etc_private_keep) {
+				if ( asprintf(&cfg.etc_private_keep, "%s,%s", cfg.etc_private_keep, ptr + 12) < 0 )
+					errExit("asprintf");
+			} else {
+				cfg.etc_private_keep = ptr + 12;
+			}
+			arg_private_etc = 1;
 		}
-		if (cfg.etc_private_keep) {
-			if ( asprintf(&cfg.etc_private_keep, "%s,%s", cfg.etc_private_keep, ptr + 12) < 0 )
-				errExit("asprintf");
-		} else {
-			cfg.etc_private_keep = ptr + 12;
-		}
-		arg_private_etc = 1;
-
+		else
+			warning_feature_disabled("private-etc");
 		return 0;
 	}
 
 	// private /opt list of files and directories
 	if (strncmp(ptr, "private-opt ", 12) == 0) {
-		if (cfg.opt_private_keep) {
-			if ( asprintf(&cfg.opt_private_keep, "%s,%s", cfg.opt_private_keep, ptr + 12) < 0 )
-				errExit("asprintf");
-		} else {
-			cfg.opt_private_keep = ptr + 12;
+		if (checkcfg(CFG_PRIVATE_OPT)) {
+			if (cfg.opt_private_keep) {
+				if ( asprintf(&cfg.opt_private_keep, "%s,%s", cfg.opt_private_keep, ptr + 12) < 0 )
+					errExit("asprintf");
+			} else {
+				cfg.opt_private_keep = ptr + 12;
+			}
+			arg_private_opt = 1;
 		}
-		arg_private_opt = 1;
-
+		else
+			warning_feature_disabled("private-opt");
 		return 0;
 	}
 
 	// private /srv list of files and directories
 	if (strncmp(ptr, "private-srv ", 12) == 0) {
-		if (cfg.srv_private_keep) {
-			if ( asprintf(&cfg.srv_private_keep, "%s,%s", cfg.srv_private_keep, ptr + 12) < 0 )
-				errExit("asprintf");
-		} else {
-			cfg.srv_private_keep = ptr + 12;
+		if (checkcfg(CFG_PRIVATE_SRV)) {
+			if (cfg.srv_private_keep) {
+				if ( asprintf(&cfg.srv_private_keep, "%s,%s", cfg.srv_private_keep, ptr + 12) < 0 )
+					errExit("asprintf");
+			} else {
+				cfg.srv_private_keep = ptr + 12;
+			}
+			arg_private_srv = 1;
 		}
-		arg_private_srv = 1;
-
+		else
+			warning_feature_disabled("private-srv");
 		return 0;
 	}
 
 	// private /bin list of files
 	if (strncmp(ptr, "private-bin ", 12) == 0) {
-		if (cfg.bin_private_keep) {
-			if ( asprintf(&cfg.bin_private_keep, "%s,%s", cfg.bin_private_keep, ptr + 12) < 0 )
-				errExit("asprintf");
-		} else {
-			cfg.bin_private_keep = ptr + 12;
+		if (checkcfg(CFG_PRIVATE_BIN)) {
+			if (cfg.bin_private_keep) {
+				if ( asprintf(&cfg.bin_private_keep, "%s,%s", cfg.bin_private_keep, ptr + 12) < 0 )
+					errExit("asprintf");
+			} else {
+				cfg.bin_private_keep = ptr + 12;
+			}
+			arg_private_bin = 1;
 		}
-		arg_private_bin = 1;
+		else
+			warning_feature_disabled("private-bin");
 		return 0;
 	}
 
@@ -1483,8 +1507,11 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 			arg_rlimit_nproc = 1;
 		}
 		else if (strncmp(ptr, "rlimit-fsize ", 13) == 0) {
-			check_unsigned(ptr + 13, "Error: invalid rlimit in profile file: ");
-			sscanf(ptr + 13, "%llu", &cfg.rlimit_fsize);
+			cfg.rlimit_fsize = parse_arg_size(ptr + 13);
+			if (cfg.rlimit_fsize == 0) {
+				perror("Error: invalid rlimit-fsize in profile file. Only use positive numbers and k, m or g suffix.");
+				exit(1);
+			}
 			arg_rlimit_fsize = 1;
 		}
 		else if (strncmp(ptr, "rlimit-sigpending ", 18) == 0) {
@@ -1493,8 +1520,11 @@ int profile_check_line(char *ptr, int lineno, const char *fname) {
 			arg_rlimit_sigpending = 1;
 		}
 		else if (strncmp(ptr, "rlimit-as ", 10) == 0) {
-			check_unsigned(ptr + 10, "Error: invalid rlimit in profile file: ");
-			sscanf(ptr + 10, "%llu", &cfg.rlimit_as);
+			cfg.rlimit_as = parse_arg_size(ptr + 10);
+			if (cfg.rlimit_as == 0) {
+				perror("Error: invalid rlimit-as in profile file. Only use positive numbers and k, m or g suffix.");
+				exit(1);
+			}
 			arg_rlimit_as = 1;
 		}
 		else {
@@ -1688,7 +1718,7 @@ void profile_read(const char *fname) {
 	}
 
 	// open profile file:
-	FILE *fp = fopen(fname, "r");
+	FILE *fp = fopen(fname, "re");
 	if (fp == NULL) {
 		fprintf(stderr, "Error: cannot open profile file %s\n", fname);
 		exit(1);
@@ -1705,13 +1735,17 @@ void profile_read(const char *fname) {
 	int lineno = 0;
 	while (fgets(buf, MAX_READ, fp)) {
 		++lineno;
+
+		// remove comments
+		char *ptr = strchr(buf, '#');
+		if (ptr)
+			*ptr = '\0';
+
 		// remove empty space - ptr in allocated memory
-		char *ptr = line_remove_spaces(buf);
+		ptr = line_remove_spaces(buf);
 		if (ptr == NULL)
 			continue;
-
-		// comments
-		if (*ptr == '#' || *ptr == '\0') {
+		if (*ptr == '\0') {
 			free(ptr);
 			continue;
 		}
@@ -1721,7 +1755,7 @@ void profile_read(const char *fname) {
 		if (strcmp(ptr, "quiet") == 0) {
 			if (is_in_ignore_list(ptr))
 				arg_quiet = 0;
-			else
+			else if (!arg_debug)
 				arg_quiet = 1;
 			free(ptr);
 			continue;
@@ -1768,9 +1802,148 @@ void profile_read(const char *fname) {
 //		else {
 //			free(ptr);
 //		}
-#ifdef HAVE_GCOV
+
 		__gcov_flush();
-#endif
 	}
 	fclose(fp);
 }
+
+char *profile_list_normalize(char *list)
+{
+	/* Remove redundant commas.
+	 *
+	 * As result is always shorter than original,
+	 * in-place copying can be used.
+	 */
+	size_t i = 0;
+	size_t j = 0;
+	int c;
+	while (list[i] == ',')
+		++i;
+	while ((c = list[i++])) {
+		if (c == ',') {
+			while (list[i] == ',')
+				++i;
+			if (list[i] == 0)
+				break;
+		}
+		list[j++] = c;
+	}
+	list[j] = 0;
+	return list;
+}
+
+char *profile_list_compress(char *list)
+{
+	size_t i;
+
+	/* Comma separated list is processed so that:
+	 * "item"  -> adds item to list
+	 * "-item" -> removes item from list
+	 * "+item" -> adds item to list
+	 * "=item" -> clear list, add item
+	 *
+	 * For example:
+	 * ,a,,,b,,,c, -> a,b,c
+	 * a,,b,,,c,a  -> a,b,c
+	 * a,b,c,-a    -> b,c
+	 * a,b,c,-a,a  -> b,c,a
+	 * a,+b,c      -> a,b,c
+	 * a,b,=c,d    -> c,d
+	 * a,b,c,=     ->
+	 */
+	profile_list_normalize(list);
+
+	/* Count items: comma count + 1 */
+	size_t count = 1;
+	for (i = 0; list[i]; ++i) {
+		if (list[i] == ',')
+			++count;
+	}
+
+	/* Collect items in an array */
+	char *in[count];
+	count = 0;
+	in[count++] = list;
+	for (i = 0; list[i]; ++i) {
+		if (list[i] != ',')
+			continue;
+		list[i] = 0;
+		in[count++] = list + i + 1;
+	}
+
+	/* Filter array: add, remove, reset, filter out duplicates */
+	for (i = 0; i < count; ++i) {
+		char *item = in[i];
+		assert(item);
+
+		size_t k;
+		switch (*item) {
+		case '-':
+			++item;
+			/* Do not include this item */
+			in[i] = 0;
+			/* Remove if already included */
+			for (k = 0; k < i; ++k) {
+				if (in[k] && !strcmp(in[k], item)) {
+					in[k] = 0;
+					break;
+				}
+			}
+			break;
+		case '+':
+			/* Allow +/- symmetry */
+			in[i] = ++item;
+			/* FALLTHRU */
+		default:
+			/* Adding empty item is a NOP */
+			if (!*item) {
+				in[i] = 0;
+				break;
+			}
+			/* Include item unless it is already included */
+			for (k = 0; k < i; ++k) {
+				if (in[k] && !strcmp(in[k], item)) {
+					in[i] = 0;
+					break;
+				}
+			}
+			break;
+		case '=':
+			in[i] = ++item;
+			/* Include non-empty item */
+			if (!*item)
+				in[i] = 0;
+			/* Remove all allready included items */
+			for (k = 0; k < i; ++k)
+				in[k] = 0;
+			break;
+		}
+	}
+
+	/* Copying back using in-place data works because the
+	 * original order is retained and no item gets longer
+	 * than what it used to be.
+	 */
+	char *pos = list;
+	for (i = 0; i < count; ++i) {
+		char *item = in[i];
+		if (!item)
+			continue;
+		if (pos > list)
+			*pos++ = ',';
+		while (*item)
+			*pos++ = *item++;
+	}
+	*pos = 0;
+	return list;
+}
+
+void profile_list_augment(char **list, const char *items)
+{
+	char *tmp = 0;
+	if (asprintf(&tmp, "%s,%s", *list ?: "", items ?: "") < 0)
+		errExit("asprintf");
+	free(*list);
+	*list = profile_list_compress(tmp);
+}
