@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2020 Firejail Authors
+ * Copyright (C) 2014-2021 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -19,6 +19,7 @@
 */
 
 #include "firejail.h"
+#include "../include/gcov_wrapper.h"
 #include "../include/seccomp.h"
 #include <sys/mman.h>
 #include <sys/mount.h>
@@ -49,7 +50,6 @@
 #include <sys/apparmor.h>
 #endif
 
-
 static int force_nonewprivs = 0;
 
 static int monitored_pid = 0;
@@ -67,7 +67,7 @@ static void sandbox_handler(int sig){
 		if (asprintf(&monfile, "/proc/%d/cmdline", monitored_pid) == -1)
 			errExit("asprintf");
 		while (monsec) {
-			FILE *fp = fopen(monfile, "r");
+			FILE *fp = fopen(monfile, "re");
 			if (!fp)
 				break;
 
@@ -162,7 +162,7 @@ static void save_nogroups(void) {
 	if (arg_nogroups == 0)
 		return;
 
-	FILE *fp = fopen(RUN_GROUPS_CFG, "w");
+	FILE *fp = fopen(RUN_GROUPS_CFG, "wxe");
 	if (fp) {
 		fprintf(fp, "\n");
 		SET_PERMS_STREAM(fp, 0, 0, 0644); // assume mode 0644
@@ -227,7 +227,7 @@ static void sandbox_if_up(Bridge *br) {
 	if (br->arg_ip_none == 1);	// do nothing
 	else if (br->arg_ip_none == 0 && br->macvlan == 0) {
 		if (br->ipsandbox == br->ip) {
-			fprintf(stderr, "Error: %d.%d.%d.%d is interface %s address.\n", PRINT_IP(br->ipsandbox), br->dev);
+			fprintf(stderr, "Error: %d.%d.%d.%d is interface %s address, exiting...\n", PRINT_IP(br->ipsandbox), br->dev);
 			exit(1);
 		}
 
@@ -245,13 +245,17 @@ static void sandbox_if_up(Bridge *br) {
 			br->ipsandbox = arp_assign(dev, br); //br->ip, br->mask);
 		else {
 			if (br->ipsandbox == br->ip) {
-				fprintf(stderr, "Error: %d.%d.%d.%d is interface %s address.\n", PRINT_IP(br->ipsandbox), br->dev);
+				fprintf(stderr, "Error: %d.%d.%d.%d is interface %s address, exiting...\n", PRINT_IP(br->ipsandbox), br->dev);
+				exit(1);
+			}
+			if (br->ipsandbox == cfg.defaultgw) {
+				fprintf(stderr, "Error: %d.%d.%d.%d is the default gateway, exiting...\n", PRINT_IP(br->ipsandbox));
 				exit(1);
 			}
 
 			uint32_t rv = arp_check(dev, br->ipsandbox);
 			if (rv) {
-				fprintf(stderr, "Error: the address %d.%d.%d.%d is already in use.\n", PRINT_IP(br->ipsandbox));
+				fprintf(stderr, "Error: the address %d.%d.%d.%d is already in use, exiting...\n", PRINT_IP(br->ipsandbox));
 				exit(1);
 			}
 		}
@@ -268,8 +272,7 @@ static void sandbox_if_up(Bridge *br) {
 
 static void chk_chroot(void) {
 	// if we are starting firejail inside some other container technology, we don't care about this
-	char *mycont = getenv("container");
-	if (mycont)
+	if (env_get("container"))
 		return;
 
 	// check if this is a regular chroot
@@ -419,7 +422,7 @@ static int ok_to_run(const char *program) {
 			return 1;
 	}
 	else { // search $PATH
-		char *path1 = getenv("PATH");
+		const char *path1 = env_get("PATH");
 		if (path1) {
 			if (arg_debug)
 				printf("Searching $PATH for %s\n", program);
@@ -463,10 +466,10 @@ static int ok_to_run(const char *program) {
 
 void start_application(int no_sandbox, int fd, char *set_sandbox_status) {
 	// set environment
-	if (no_sandbox == 0) {
+	if (no_sandbox == 0)
 		env_defaults();
-		env_apply();
-	}
+	env_apply_all();
+
 	// restore original umask
 	umask(orig_umask);
 
@@ -475,24 +478,10 @@ void start_application(int no_sandbox, int fd, char *set_sandbox_status) {
 		printf("LD_PRELOAD=%s\n", getenv("LD_PRELOAD"));
 	}
 
-	//****************************************
-	// audit
-	//****************************************
-	if (arg_audit) {
-		assert(arg_audit_prog);
-
-#ifdef HAVE_GCOV
-		__gcov_dump();
-#endif
-		seccomp_install_filters();
-		if (set_sandbox_status)
-			*set_sandbox_status = SANDBOX_DONE;
-		execl(arg_audit_prog, arg_audit_prog, NULL);
-	}
 	//****************************************
 	// start the program without using a shell
 	//****************************************
-	else if (arg_shell_none) {
+	if (arg_shell_none) {
 		if (arg_debug) {
 			int i;
 			for (i = cfg.original_program_index; i < cfg.original_argc; i++) {
@@ -515,9 +504,8 @@ void start_application(int no_sandbox, int fd, char *set_sandbox_status) {
 			exit(1);
 		}
 
-#ifdef HAVE_GCOV
 		__gcov_dump();
-#endif
+
 		seccomp_install_filters();
 
 		if (set_sandbox_status)
@@ -571,9 +559,8 @@ void start_application(int no_sandbox, int fd, char *set_sandbox_status) {
 		if (!arg_command && !arg_quiet)
 			print_time();
 
-#ifdef HAVE_GCOV
 		__gcov_dump();
-#endif
+
 		seccomp_install_filters();
 
 		if (set_sandbox_status)
@@ -590,12 +577,12 @@ void start_application(int no_sandbox, int fd, char *set_sandbox_status) {
 }
 
 static void enforce_filters(void) {
+	fmessage("\n** Warning: dropping all Linux capabilities and setting NO_NEW_PRIVS prctl **\n\n");
 	// enforce NO_NEW_PRIVS
 	arg_nonewprivs = 1;
 	force_nonewprivs = 1;
 
 	// disable all capabilities
-	fmessage("\n**     Warning: dropping all Linux capabilities     **\n\n");
 	arg_caps_drop_all = 1;
 
 	// drop all supplementary groups; /etc/group file inside chroot
@@ -796,14 +783,18 @@ int sandbox(void* sandbox_arg) {
 			exit(rv);
 	}
 
-	// need ld.so.preload if tracing or seccomp with any non-default lists
-	bool need_preload = arg_trace || arg_tracelog || arg_seccomp_postexec;
+#ifdef HAVE_FORCE_NONEWPRIVS
+	bool always_enforce_filters = true;
+#else
+	bool always_enforce_filters = false;
+#endif
 	// for --appimage, --chroot and --overlay* we force NO_NEW_PRIVS
 	// and drop all capabilities
-	if (getuid() != 0 && (arg_appimage || cfg.chrootdir || arg_overlay)) {
+	if (getuid() != 0 && (arg_appimage || cfg.chrootdir || arg_overlay || always_enforce_filters))
 		enforce_filters();
-		need_preload = arg_trace || arg_tracelog;
-	}
+
+	// need ld.so.preload if tracing or seccomp with any non-default lists
+	bool need_preload = arg_trace || arg_tracelog || arg_seccomp_postexec;
 
 	// trace pre-install
 	if (need_preload)
@@ -835,10 +826,16 @@ int sandbox(void* sandbox_arg) {
 #endif
 		fs_basic_fs();
 
+	//****************************
+	// appimage
+	//****************************
+	appimage_mount();
+
 	//****************************
 	// private mode
 	//****************************
 	if (arg_private) {
+		EUID_USER();
 		if (cfg.home_private) {	// --private=
 			if (cfg.chrootdir)
 				fwarning("private=directory feature is disabled in chroot\n");
@@ -857,6 +854,7 @@ int sandbox(void* sandbox_arg) {
 		}
 		else // --private
 			fs_private();
+		EUID_ROOT();
 	}
 
 	if (arg_private_dev)
@@ -970,11 +968,35 @@ int sandbox(void* sandbox_arg) {
 		else if (arg_overlay)
 			fwarning("private-etc feature is disabled in overlay\n");
 		else {
-			fs_private_dir_list("/etc", RUN_ETC_DIR, cfg.etc_private_keep);
-			fs_private_dir_list("/usr/etc", RUN_USR_ETC_DIR, cfg.etc_private_keep); // openSUSE
+			/* Current /etc/passwd and /etc/group files are bind
+			 * mounted filtered versions of originals. Leaving
+			 * them underneath private-etc mount causes problems
+			 * in devices with older kernels, e.g. attempts to
+			 * update the real /etc/passwd file yield EBUSY.
+			 *
+			 * As we do want to retain filtered /etc content:
+			 * 1. duplicate /etc content to RUN_ETC_DIR
+			 * 2. unmount bind mounts from /etc
+			 * 3. mount RUN_ETC_DIR at /etc
+			 */
+			timetrace_start();
+			fs_private_dir_copy("/etc", RUN_ETC_DIR, cfg.etc_private_keep);
+
+			if (umount2("/etc/group", MNT_DETACH) == -1)
+				fprintf(stderr, "/etc/group: unmount: %s\n", strerror(errno));
+			if (umount2("/etc/passwd", MNT_DETACH) == -1)
+				fprintf(stderr, "/etc/passwd: unmount: %s\n", strerror(errno));
+
+			fs_private_dir_mount("/etc", RUN_ETC_DIR);
+			fmessage("Private /etc installed in %0.2f ms\n", timetrace_end());
+
 			// create /etc/ld.so.preload file again
 			if (need_preload)
 				fs_trace_preload();
+
+			// openSUSE configuration is split between /etc and /usr/etc
+			// process private-etc a second time
+			fs_private_dir_list("/usr/etc", RUN_USR_ETC_DIR, cfg.etc_private_keep);
 		}
 	}
 
@@ -997,7 +1019,7 @@ int sandbox(void* sandbox_arg) {
 		// disable /dev/snd
 		fs_dev_disable_sound();
 	}
-	else if (!arg_noautopulse)
+	else if (!arg_keep_config_pulse)
 		pulseaudio_init();
 
 	if (arg_no3d)
@@ -1015,23 +1037,14 @@ int sandbox(void* sandbox_arg) {
 	if (arg_novideo)
 		fs_dev_disable_video();
 
-	//****************************
-	// install trace
-	//****************************
-	if (need_preload)
-		fs_trace();
+	if (arg_noinput)
+		fs_dev_disable_input();
 
 	//****************************
 	// set dns
 	//****************************
 	fs_resolvconf();
 
-	//****************************
-	// fs post-processing
-	//****************************
-	fs_logger_print();
-	fs_logger_change_owner();
-
 	//****************************
 	// start dhcp client
 	//****************************
@@ -1080,6 +1093,12 @@ int sandbox(void* sandbox_arg) {
 	// save original umask
 	save_umask();
 
+	//****************************
+	// fs post-processing
+	//****************************
+	fs_logger_print();
+	fs_logger_change_owner();
+
 	//****************************
 	// set security filters
 	//****************************
@@ -1137,13 +1156,21 @@ int sandbox(void* sandbox_arg) {
 	fs_remount(RUN_SECCOMP_DIR, MOUNT_READONLY, 0);
 	seccomp_debug();
 
+	//****************************
+	// install trace - still need capabilities
+	//****************************
+	if (need_preload)
+		fs_trace();
+
+	//****************************
+	// continue security filters
+	//****************************
 	// set capabilities
 	set_caps();
 
 	//****************************************
 	// relay status information to join option
 	//****************************************
-
 	char *set_sandbox_status = create_join_file();
 
 	//****************************************
@@ -1204,7 +1231,6 @@ int sandbox(void* sandbox_arg) {
 	//****************************************
 	// set cpu affinity
 	//****************************************
-
 	if (cfg.cpus)
 		set_cpu_affinity();
 
