@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2020 Firejail Authors
+ * Copyright (C) 2014-2021 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -36,7 +36,7 @@ static int __attribute__((noreturn)) sbox_do_exec_v(unsigned filtermask, char *
 	int env_index = 0;
 	char *new_environment[256] = { NULL };
 	// preserve firejail-specific env vars
-	char *cl = getenv("FIREJAIL_FILE_COPY_LIMIT");
+	const char *cl = env_get("FIREJAIL_FILE_COPY_LIMIT");
 	if (cl) {
 		if (asprintf(&new_environment[env_index++], "FIREJAIL_FILE_COPY_LIMIT=%s", cl) == -1)
 			errExit("asprintf");
@@ -120,7 +120,7 @@ static int __attribute__((noreturn)) sbox_do_exec_v(unsigned filtermask, char *
 			// handle X32 ABI
 			BPF_JUMP(BPF_JMP + BPF_JGE + BPF_K, X32_SYSCALL_BIT, 1, 0),
 			BPF_JUMP(BPF_JMP + BPF_JGE + BPF_K, 0, 1, 0),
-			RETURN_ERRNO(EPERM),
+			KILL_OR_RETURN_ERRNO,
 #endif
 
 		// syscall list
@@ -203,15 +203,16 @@ static int __attribute__((noreturn)) sbox_do_exec_v(unsigned filtermask, char *
 		}
 	}
 
-	if (filtermask & SBOX_ROOT) {
+	if (filtermask & SBOX_USER)
+		drop_privs(1);
+	else if (filtermask & SBOX_ROOT) {
 		// elevate privileges in order to get grsecurity working
 		if (setreuid(0, 0))
 			errExit("setreuid");
 		if (setregid(0, 0))
 			errExit("setregid");
 	}
-	else if (filtermask & SBOX_USER)
-		drop_privs(1);
+	else assert(0);
 
 	if (arg[0]) { // get rid of scan-build warning
 		int fd = open(arg[0], O_PATH | O_CLOEXEC);
@@ -247,7 +248,9 @@ int sbox_run(unsigned filtermask, int num, ...) {
 	va_start(valist, num);
 
 	// build argument list
-	char **arg = malloc((num + 1) * sizeof(char *));
+	char **arg = calloc(num + 1, sizeof(char *));
+	if (!arg)
+		errExit("calloc");
 	int i;
 	for (i = 0; i < num; i++)
 		arg[i] = va_arg(valist, char *);
@@ -262,7 +265,6 @@ int sbox_run(unsigned filtermask, int num, ...) {
 }
 
 int sbox_run_v(unsigned filtermask, char * const arg[]) {
-	EUID_ROOT();
 	assert(arg);
 
 	if (arg_debug) {
@@ -282,6 +284,7 @@ int sbox_run_v(unsigned filtermask, char * const arg[]) {
 	if (child < 0)
 		errExit("fork");
 	if (child == 0) {
+		EUID_ROOT();
 		sbox_do_exec_v(filtermask, arg);
 	}
 
@@ -290,7 +293,7 @@ int sbox_run_v(unsigned filtermask, char * const arg[]) {
 		errExit("waitpid");
 	}
 	if (WIFEXITED(status) && WEXITSTATUS(status) != 0) {
-		fprintf(stderr, "Error: failed to run %s\n", arg[0]);
+		fprintf(stderr, "Error: failed to run %s, exiting...\n", arg[0]);
 		exit(1);
 	}
 
