@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2020 Firejail Authors
+ * Copyright (C) 2014-2021 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -52,7 +52,7 @@ static void my_handler(int s){
 
 	if (terminal_set)
 		tcsetattr(0, TCSANOW, &tlocal);
-	exit(0);
+	_exit(0);
 }
 
 // find the second child process for the specified pid
