@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2020 Firejail Authors
+ * Copyright (C) 2014-2021 Firejail Authors
  *
  * This file is part of firejail project
  *
@@ -23,13 +23,16 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
+// todo: resolve overlap with masked_lib_dirs[] array from fs_lib.c
 const char * const default_lib_paths[] = {
 	"/usr/lib/x86_64-linux-gnu",	// Debian & friends
 	"/lib/x86_64-linux-gnu",		// CentOS, Fedora
+	"/usr/lib64",
+	"/lib64",
 	"/usr/lib",
 	"/lib",
-	"/lib64",
 	LIBDIR,
+	"/usr/local/lib64",
 	"/usr/local/lib",
 	"/usr/lib/x86_64-linux-gnu/mesa", // libGL.so is sometimes a symlink into this directory
 	"/usr/lib/x86_64-linux-gnu/mesa-egl", // libGL.so is sometimes a symlink into this directory
@@ -47,7 +50,7 @@ int is_lib_64(const char *exe) {
 	unsigned char buf[EI_NIDENT];
 	ssize_t len = 0;
 	while (len < EI_NIDENT) {
-		ssize_t sz = read(fd, buf, EI_NIDENT);
+		ssize_t sz = read(fd, buf + len, EI_NIDENT - len);
 		if (sz <= 0)
 			goto doexit;
 		len += sz;
